#!/usr/bin/env python
from objectname import objectname
from public import public
from tolist import tolist


def err_msg(func, args, pos, required_types):
    name = objectname(func, fullname=True)
    lines = [
        "%s() argument #%s is not instance of %s" %
        (name, pos, required_types)]
    lines += ["args: %s" % str(list(args))]
    lines += ["arg#%s: %s" % (pos, args[pos])]
    msg = "\n".join(lines)
    raise TypeError(msg)


@public
def accepts(*types):
    def check_accepts(f):
        # assert len(types) == f.func_code.co_argcount
        def new_f(*args, **kwargs):
            for pos, (arg, required_types) in enumerate(zip(args, types), 0):
                # isinstance() arg 2 must be a type or tuple of types
                # fix: type(None)
                if not isinstance(arg, required_types):
                    msg = err_msg(f, args, pos, required_types)
                    raise TypeError(msg)
            return f(*args, **kwargs)
        try:  # python2
            new_f.func_name = f.func_name
        except AttributeError:  # python3
            new_f.__name__ = f.__name__
        return new_f
    return check_accepts



#!/usr/bin/env python
from distutils.core import setup  # Python Standard Library
# pypi.org/project/setuptools/ (python3.2 not supported)
# setup.cfg support: setuptools 30.3.0+
try:
    from configparser import ConfigParser  # python 3+
except ImportError:
    from ConfigParser import ConfigParser  # python 2.x

# ~/.pydistutils.cfg
# path/to/project/README.rst
# path/to/project/setup.cfg
# path/to/project/setup.py


def get_item(key, value):
    if value and value[0] == "\n":  # array (first line empty)
        value = list(filter(None, value.splitlines()))
    # 1) Python 2.x ConfigParser stripping blank lines in multiline value
    # 2) ConfigParser breaks tab/space indentation (use README.rst)
    if key == "description-file":  # ./README.rst
        key = "long_description"
        try:
            value = open(value).read()
        except:
            value = None
    return key, value


def read_configuration(path):
    config = ConfigParser()
    config.read(path)
    result = dict()
    for section in config.sections():
        for key, value in config.items(section):
            key, value = get_item(key, value)
            result[key] = value
    return result


path = __file__.replace("setup.py", "setup.cfg")
setup_cfg_data = read_configuration(path)
setup(**setup_cfg_data)
